
#include <freertos/FreeRTOS.h>
#include <freertos/task.h>
#include <driver/gpio.h>
#include "fsm.h"

const unsigned PERIODO = 1000;
const gpio_num_t BTN = 3;
const gpio_num_t LED = 8;
volatile bool button_pressed_event = false;

typedef struct {
    fsm_t parent;
    int segundos;
} pasillo_fsm_t;


static pasillo_fsm_t* pasillo_fsm(fsm_t* fsm) {
    return (pasillo_fsm_t*) fsm;
}

static int button_pressed(fsm_t* fsm) {
    printf("button_pressed: %d\n", gpio_get_level(BTN));
    return button_pressed_event;
}

static int reloj_1s(fsm_t* fsm) {
    return 1;
}

static int timer_expired(fsm_t* fsm) {
    pasillo_fsm_t* p = pasillo_fsm(fsm);
    printf("timer_expired: %d\n", p->segundos);
    return p->segundos <= 0;
}

static void lampara_on (fsm_t* fsm)
{
    pasillo_fsm_t* p = pasillo_fsm(fsm);
    gpio_set_level(LED, 1);
    printf("ON\n");
    p->segundos = 20;
    button_pressed_event = false;
}

static void lampara_off (fsm_t* fsm)
{
    gpio_set_level(LED, 0);
    printf("OFF\n");
}

static void dec_segundos (fsm_t* fsm)
{
    pasillo_fsm_t* p = pasillo_fsm(fsm);
    printf("dec_segundos %d -> ", p->segundos);
    p->segundos = p->segundos - 1;
    printf("%d\n", p->segundos);
}

static void button_isr_handler(void* arg) 
{ 
    static TickType_t lastEvent;
    TickType_t current = xTaskGetTickCountFromISR();
    if (current - lastEvent > 500 / portTICK_PERIOD_MS) { // debouncing 500ms
        button_pressed_event = true;
    }
    lastEvent = current;
}

// constructor

fsm_t* pasillo_fsm_new (void)
{
    static fsm_trans_t tt[] = {
        {  0, button_pressed, 1, lampara_on },
        {  1, button_pressed, 1, lampara_on },
        {  1, timer_expired,  0, lampara_off},
        {  1, reloj_1s,       1, dec_segundos },
        { -1, NULL, -1, NULL },
    };
    pasillo_fsm_t* this = (pasillo_fsm_t*) malloc(sizeof(pasillo_fsm_t));
    fsm_init(&this->parent, tt);
    this->segundos = 0;
    return &this->parent;
}


void app_main(void)
{
    gpio_install_isr_service(0);
    gpio_reset_pin(BTN);
    gpio_set_direction(LED, GPIO_MODE_OUTPUT);
    gpio_set_direction(BTN, GPIO_MODE_INPUT);
    gpio_pullup_en(BTN);
    gpio_set_intr_type(BTN, GPIO_INTR_NEGEDGE);
    gpio_isr_handler_add(BTN, button_isr_handler, NULL);

    TickType_t last = xTaskGetTickCount();
    fsm_t* fsm = pasillo_fsm_new();
    for (;;) {
        fsm_update(fsm);
        vTaskDelayUntil(&last, pdMS_TO_TICKS(PERIODO));
    }
}
